# File: `Projects/veritell-langchain/src/veritell_langchain/__init__.py`

from __future__ import annotations

from importlib.metadata import PackageNotFoundError, version

from .errors import (
    RateLimitedError,
    ServiceUnavailableError,
    VeritellAPIError,
    VeritellAuthError,
    VeritellConnectionError,
    VeritellError,
    VeritellRequestError,
    VeritellServerError,
    VeritellTimeoutError,
    VeritellValidationError,
)
from .evaluator import VeritellEvaluator
from .models import SCHEMA_VERSION, VeritellEvaluationResult, VeritellStreamEvent

try:  # pragma: no cover
    __version__ = version("veritell-langchain")
except PackageNotFoundError:  # pragma: no cover
    __version__ = "0.0.0"

__all__ = [
    "SCHEMA_VERSION",
    "RateLimitedError",
    "ServiceUnavailableError",
    "VeritellAPIError",
    "VeritellAuthError",
    "VeritellConnectionError",
    "VeritellError",
    "VeritellEvaluationResult",
    "VeritellEvaluator",
    "VeritellRequestError",
    "VeritellServerError",
    "VeritellStreamEvent",
    "VeritellTimeoutError",
    "VeritellValidationError",
    "__version__",
]
